<?php

	/*
		Project_CMS

		Library for article management
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	

	class cms_article
	{
		var $articleData;
		
		var $text;
		var $author;
		var $changeLog;
		var $attachments;
		var $votings;
		var $locked;
				
		function cms_article($name)
		{
			// Load article
			$this->articleData 			= new cms_data_storage(CMS_DATA_PATH.'/article_'.urlencode($name));
			$this->changeLog 			= &$this->articleData->data['changeLog'];
			$this->author 				= &$this->articleData->data['author'];		
			$this->text 				= &$this->articleData->data['text'];
			$this->attachments			= &$this->articleData->data['attachments'];
			$this->votings				= &$this->articleData->data['votings'];
			$this->locked				= &$this->articleData->data['locked'];
		}
				
		function updateText($newText, $user, $comment = '')
		{
			if(!is_array($this->changeLog))
			{
				// Create new article
				$this->author 		= $user;
				$dummyChange		= array('user' => $user, 'comment' => cms_functions::string('libarticles.new'),
											 'patch' => '', 'time' => time());
				$this->changeLog 	= array($dummyChange);
			}
			else 
			{
				// Update data and create history entry
				$change 			= &$this->changeLog[];
				$change['user']		= $user;
				$change['comment']	= $comment;
				$change['patch']	= cms_versioning::diff($newText, $this->text);
				$change['time']		= time();
				
				// Check the patch for bugs
				$checkOne = cms_versioning::patch($newText, $change['patch']);
				$checkTwo = str_replace(array("\r\n", "\r"), "\n", $this->text);
				if($checkOne != $checkTwo)
					trigger_error(cms_functions::string('edit.patchfailure'), E_USER_ERROR);
			}
			
			$this->text			= $newText;
			$this->articleData->save();
		}
		
		/*
			User ratings
		*/
		function getVotingsState()
		{
			return $this->votings;
		}
		
		function setVotingsState($state)
		{
			$this->votings = $state ? true : false;
			$this->articleData->save();
		}
		
		/*
			Lock
		*/
		function getLockedState()
		{
			return $this->locked;
		}
		
		function setLockedState($state)
		{
			$this->locked = $state ? true : false;
			$this->articleData->save();
		}
					
		/*
			Changes
		*/
		function clearChanges()
		{
			$dummyChange		= array('user' => $author, 'comment' => cms_functions::string('libarticles.new'),
										'patch' => '', 'time' => time());
			$this->changeLog 	= array($dummyChange);
			$this->articleData->save();
		}
		
		function getVersion($versionID = false)
		{
			// Load the newest version
			if($versionID === false) return $this->text;
			if(!isset($this->changeLog[$versionID])) return $this->text;
			
			// Load older versions which
			// means to apply patches
			$text 		= $this->text;
			$versioning = array_reverse($this->changeLog, true);
			foreach ($versioning as $current_versionID => $change)
			{
				$text = cms_versioning::patch($text, $change['patch']);
				if($current_versionID == $versionID) return $text;
			}
			return false;
		}
		
		function getChangeLog()
		{
			return $this->changeLog;
		}
		
		/*
			Attachments
		*/
		function newAttachment($name, $fileName, $insertLink)
		{
			$this->attachments[$name] = array('file' => $fileName, 'link' => $insertLink);
			$this->articleData->save();
		}
		
		function deleteAttachment($name)
		{
			unset($this->attachments[$name]);
			$this->articleData->save();
		}
		
		function getAttachments()
		{
			return $this->attachments;
		}
	}
	
	// A PHP version of the unix tools diff and patch
	class cms_versioning
	{
		function html_diff($source, $new_version)
		{
			$source 		= explode("\n", str_replace(array("\n\r", "\r"), "\n", $source));			
			$new_version 	= explode("\n", str_replace(array("\n\r", "\r"), "\n", $new_version));
			$retVal			= '';
			
			$sourcePos = $newPos = 0;
			
			do 
			{
				// Lines are identical
				if($source[$sourcePos] == $new_version[$newPos] && isset($new_version[$newPos]) && isset($source[$sourcePos]))
				{
					$retVal .= str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($source[$sourcePos]))).'<br/>';
					$sourcePos++; $newPos++;
					continue;
				}
				
				// A line was inserted
				if(($source[$sourcePos] == $new_version[$newPos + 1] && isset($new_version[$newPos + 1])) |! isset($source[$sourcePos]))
				{
					$retVal .= '<ins>'.str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($new_version[$newPos]))).'<br/></ins>';
					$newPos++;
					continue;
				}
				
				// Multiple lines were inserted
				if(($newLength = array_search($source[$sourcePos], array_slice($new_version, $newPos))) !== false && isset($new_version[$newPos + 1]))
				{
						$finalPos = $newPos + $newLength;
						while($newPos < $finalPos)
						{
							$retVal .= '<ins>'.str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($new_version[$newPos++]))).'<br/></ins>';
						}
							
						continue;
				}
				
				// A line was changed
				if($source[$sourcePos + 1] == $new_version[$sourcePos + 1] &&
					isset($new_version[$newPos]) && isset($source[$sourcePos]) &&
					(isset($new_version[$sourcePos + 1]) == isset($source[$sourcePos + 1])))
				{
					$retVal .= '<del>'.str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($source[$sourcePos]))).'<br/></del>';
					$retVal .= '<ins>'.str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($new_version[$newPos]))).'<br/></ins>';
					
					$sourcePos++; $newPos++;
				}
				
				// A line was removed
				if(($deletedTo = array_search($new_version[$newPos], array_slice($source, $sourcePos), true)) !== false |!
					isset($new_version[$newPos+1]))
				{
					if($deletedTo !== false)
						 $deletedTo += $sourcePos - 1;
					else $deletedTo = count($source) - 1;
					
					for(; $sourcePos<=$deletedTo; $sourcePos++)
					{
						$retVal .= '<del>'.str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($source[$sourcePos]))).'<br/></del>';
					}
						
					continue;
				}
				
				// When unsure, mark the line as "changed"
				$retVal .= '<del>'.str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($source[$sourcePos]))).'<br/></del>';
				$retVal .= '<ins>'.str_replace(array("\t", "  "), '&nbsp; ', htmlspecialchars(rtrim($new_version[$newPos]))).'<br/></ins>';
				$sourcePos++; $newPos++;
				continue;
			}
			while(isset($source[$sourcePos]) || isset($new_version[$newPos]));
			
			return '<div class="changeDiffer">'.$retVal.'</div>';
		}

		function diff($source, $new_version)
		{
			$source 		= explode("\n", str_replace(array("\r\n", "\r"), "\n", $source));			
			$new_version 	= explode("\n", str_replace(array("\r\n", "\r"), "\n", $new_version));
			$diff 			= array();
			$sourcePos = $newPos = 0;
			
			do 
			{
				// Lines are identical
				if($source[$sourcePos] == $new_version[$newPos] && isset($new_version[$newPos]) && isset($source[$sourcePos]))
				{
					// Uncomment this to save lines that haven't changed
					// $diff[$sourcePos] .= '= '.$sourcePos.' '.$source[$sourcePos]."\n";
					$sourcePos++; $newPos++;
					continue;
				}
				
				// A line was inserted
				if(($source[$sourcePos] == $new_version[$newPos + 1] && isset($new_version[$newPos + 1])) |! isset($source[$sourcePos]))
				{
					$diff[$sourcePos] .= '+ '.($sourcePos-1).' '.$new_version[$newPos]."\n";
					$newPos++;
					continue;
				}
				
				// Multiple lines were inserted
				if(($newLength = array_search($source[$sourcePos], array_slice($new_version, $newPos))) !== false && isset($new_version[$newPos + 1]))
				{
						$finalPos = $newPos + $newLength;
						while($newPos < $finalPos)
						{
							$diff[$sourcePos] .= '+ '.($sourcePos-1).' '.$new_version[$newPos++]."\n";
						}
							
						continue;
				}
				
				// A line was changed
				if($source[$sourcePos + 1] == $new_version[$sourcePos + 1] &&
					isset($new_version[$newPos]) && isset($source[$sourcePos]) &&
					(isset($new_version[$sourcePos + 1]) == isset($source[$sourcePos + 1])))
				{
					$diff[$sourcePos] .= '~ '.$sourcePos.' '.$new_version[$newPos]."\n";
					$sourcePos++; $newPos++;
				}
				
				// A line was removed
				if(($deletedTo = array_search($new_version[$newPos], array_slice($source, $sourcePos), true)) !== false |!
					isset($new_version[$newPos+1]))
				{
					if($deletedTo !== false)
						 $deletedTo += $sourcePos - 1;
					else $deletedTo = count($source) - 1;
					
					for(; $sourcePos<=$deletedTo; $sourcePos++)
					{
						$diff[$sourcePos] .= '- '.$sourcePos.' -'."\n";
						// For debugging purposes save the text:
						// $diff[$sourcePos] .= '- '.$sourcePos.' '.$source[$sourcePos]."\n";
					}
					
					continue;
				}
				
				// When unsure, mark the line as "changed"
				$diff[$sourcePos] .= '~ '.$sourcePos.' '.$new_version[$newPos]."\n";
				$sourcePos++; $newPos++;
				continue;
			}
			while(isset($source[$sourcePos]) || isset($new_version[$newPos]));
			
			return implode('', $diff);
		}
		
		function patch($source, $patch)
		{
			$source = str_replace(array("\r\n", "\r"), "\n", explode("\n", $source));
			$patch 	= str_replace(array("\r\n", "\r"), "\n", explode("\n", $patch));
			
			foreach($patch as $patchline)
			{
				list($action, $line, $text) = explode(' ', $patchline, 3);
				
				if($action == null)
					break;
					
				switch($action)
				{
					case '=': 
						// Nothing, cause nothing changed ;)
						break;
						
					case '+':
						// Insert
						if($source[$line] === false |! isset($source[$line]))
							$source[$line] = $text;
						else
							$source[$line] = $source[$line]."\n".$text;
						break;
						
					case '-':
						// Delete
						$source[$line] = false;
						break;
						
					case '~':
						// Change
						$source[$line] = $text;
						break;
						
					default:
						trigger_error(sprintf(cms_functions::string('libarticles.patchfailed'), $patchline), E_USER_ERROR);
				}
			}
			
			foreach($source as $key => $value)
				if($value === false)
					unset($source[$key]);
					
			return implode("\n", $source);
		}
	}
?>