<?php

	/*
		Project_CMS

		Library for page caching
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	

	class cms_caching
	{
		var $currentPage;
		var $dontCache;
		
		function cms_caching()
		{
			// Create an id for the current page
			// Maybe its already saved in the cache so that we don't have to parse it?!
			$this->currentPage = basename($_SERVER['PHP_SELF']).'-'.urlencode($_SERVER['QUERY_STRING']).'.html';
			
			// If POST Data was sent, never cache the page
			if(!empty($_POST))
				return;
			
			// If the current page exists in the cache directory, return the cached version
			if(file_exists(CMS_DATA_PATH.'/cache/'.$this->currentPage))
			{
				$cached_data_file = file_get_contents(CMS_DATA_PATH.'/cache/'.$this->currentPage);
				// Output a correct header (For RSS feet)
				if(strncmp('<?xml', $cached_data_file, 5) === 0)
					header('Content-type: text/xml');
				die($cached_data_file);
			}
			
			// Start caching
			ob_start(array('cms_caching', 'ob_handler'));
		}
		
		// The output handler creates the cache
		function ob_handler($data)
		{
			// This function gets called statically, so get a $this reference
			if($this)
				$thisPtr = &$this;
			else
				$thisPtr = &$GLOBALS['cms_cache'];	
			
			// If the option is set, remove whitespace
			// It's a bit crappy to put this into a caching library,
			// but a second outputbuffer would slow the page down...
			if(CMS_STRIP_WHITESPACE === true)
				$data = cms_functions::html_strip_whitespace($data);
			
			if($thisPtr->dontCache)
				return $data;
				
			if(CMS_ACTIVATE_CACHING !== true)
				return $data;
			
			file_put_contents(CMS_DATA_PATH.'/cache/'.$thisPtr->currentPage, $data);
			return $data;
		}
		
		// Delete specific pages from the cache
		function clear_cache($mask)
		{
			foreach(cms_functions::get_dir(CMS_DATA_PATH.'/cache/') as $cached_file)
			{
				// Do a failsafe urldecode
				$decoded_cached_file = $cached_file;
				while($decoded_cached_file != urldecode($decoded_cached_file)) $decoded_cached_file = urldecode($decoded_cached_file);
				
				// ompare the strings and delete the cached data
				if($mask == '' || stripos($decoded_cached_file, $mask) !== false)
					unlink(CMS_DATA_PATH.'/cache/'.$cached_file);
			}
		}
		
		// If this function is called, the current page won't get cached
		function set_dont_cache_flag()
		{
			$this->dontCache = true;
		}
	}
	
	// Init class
	$cms_cache = new cms_caching();
?>