<?php

	/*
		Project_CMS

		Library for data management/saving/loading
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	

	class cms_data_storage
	{
		var $fileName;
		var $lockID;
		
		var $data;
		
		function cms_data_storage($fileName)
		{
			$this->fileName = $fileName;
			$this->reload();
		}
		
		// Reload data
		function reload()
		{
			if(file_exists($this->fileName))
			{
				$this->lockFile();
				$this->data 	= unserialize(file_get_contents($this->fileName));
				$this->unlockFile();
			}
			else 
				$this->data		= array();
		}
		
		// Save data
		function save()
		{
			$this->lockFile();
			$fileHandle = fopen($this->fileName, 'wb');
			if(!$fileHandle)
				trigger_error(sprintf(cms_functions::string('libdata.openfailed'), $this->fileName), E_USER_ERROR);
			fwrite($fileHandle, serialize($this->data));
			fclose($fileHandle);
			$this->unlockFile();
		}
		
		// Lock file for any access
		function lockFile()
		{
			if(is_resource($this->lockID))
				trigger_error(cms_functions::string('libdata.filelocked'), E_USER_WARNING);
				
			$this->lockID = fopen($this->fileName.'.lock', 'wb');
			flock($this->lockID, LOCK_EX);
		}
		
		// Unlock the file
		function unlockFile()
		{
			if(!is_resource($this->lockID))
				trigger_error(cms_functions::string('libdata.unlockfailed'), E_USER_WARNING);
				
			flock($this->lockID, LOCK_UN);
			fclose($this->lockID);
			unlink($this->fileName.'.lock');
			
			$this->lockID = null;
		}
	}
?>