<?php

	/*
		Project_CMS

		Library for logging purposes
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	

	class cms_eventlog 
	{
		var $logFile;
		var $logData;
		var $lastLogSent;
		
		function cms_eventlog()
		{
			$this->logFile 		= new cms_data_storage(CMS_DATA_PATH.'/.log');
			$this->logData 		= &$this->logFile->data['log'];
			$this->lastLogSent 	= &$this->logFile->data['lastSent'];
			
			// If anything chancged yesterday (of before) and the
			// functionality is activated, send
			// an email to the pages administrator
			if(CMS_ADMIN_EMAIL != '' && is_array($this->logData))
			{
				$latestLog = array_pop(array_values($this->logData));
				
				// $this->lastLogSent is always the time when the last logentry was sent
				// Send only one email per day, but only if there are new lines of log
				// Plus, the log has to be older than two hours
				if(date('d#m#Y') != date('d#m#Y', $this->lastLogSent) &&
				   $latestLog['time'] > $this->lastLogSent &&
				   $latestLog['time'] < strtotime('-2 hours'))
				{					
					// Create a link to the pagelog
					$url = $GLOBALS['cms_base_url'].cms_functions::generate_url(array('mode' => 'eventlog'));
					
					// Create a body for an email
					$mailBody = sprintf(cms_functions::string('libeventlog.mailbody'), $url);
					
					foreach($this->getLog() as $logLine)
						if($logLine['time'] <= $this->lastLogSent)
							break;
						else 
							$mailBody .= '- ['.$logLine['user'].' '.date(cms_functions::string('generic.longdate'), $logLine['time']).'] '.$logLine['logline']."\n";
						
					// Send the mail
					mail(CMS_ADMIN_EMAIL, cms_functions::string('libeventlog.mailsubject'), $mailBody);
					
					// Set the last-sent date of the log
					$this->lastLogSent = time();
					$this->logFile->save();
				}
			}
		}
		
		function write_log($text, $link)
		{
			$this->logData[] = array(
				'ip' => $_SERVER['REMOTE_ADDR'],
				'user' => $GLOBALS['cms_users']->currentUserNick,
				'logline' => $text,
				'time' => time(),
				'link' => $link);
			$this->logFile->save();
		}
		
		function getLog()
		{
			if(!$this->logData) return array();
			return array_reverse($this->logData);
		}
		
		function clearLog()
		{
		if($this->logData)
			foreach($this->logData as $key => $data)
				if($data['ip'] != $_SERVER['REMOTE_ADDR'] || $GLOBALS['cms_users']->currentUserNick === CMS_ADMIN_USER)
					unset($this->logData[$key]);
			$this->logFile->save();
		}
	}
	
	// Init class
	$cms_eventlog = new cms_eventlog();
?>