<?php

	/*
		Project_CMS

		Various functions
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	

	class cms_functions
	{
		// Validate HTML
		// As long as theres no uniform PHP modell (see DOMDocument in PHP4/5)
		// I'll do that using an external validator
		// Currently HTMLTidy and htmlhelp.com are supported.
		function validate_input($html_text, $charset = 'iso-8859-1')
		{
			// This could last a long time.. :)
			if(!ini_get('safe_mode'))
				@set_time_limit(120);
							
			// Create a html header and footer around the input
			$validator_input =
				'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" '.
				'"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">'.
				'<html xmlns="http://www.w3.org/1999/xhtml">'.
				'<head><meta http-equiv="Content-Type" content="text/html; charset='.$charset.'"/>'.
				'<title>validation</title></head><body><div>'."\r\n\r\n".$html_text."\r\n\r\n".'</div></body></html>';

			// Validate the input
			// Check if the tidy extension is available
			if(class_exists('tidy'))
			{
				$syntaxCheck = new tidy();
				$syntaxCheck->parseString($validator_input);
				
				if($syntaxCheck->errorBuffer != '')
				{
					$input_lines = explode("\n", $validator_input);
					array_unshift($input_lines, '');
					
					$errors = '<ul>';
					foreach(explode("\n", $syntaxCheck->errorBuffer) as $error)
					{
						// Skip some errors which are not actually critical
						foreach(array(
							'nested emphasis') as $acceptable_error)
							if(strpos($error, $acceptable_error) !== false)
								$error = '';
								
						if($error != '')
						{	
							// Output error
							if(preg_match('/line ([0-9]+) column ([0-9]+)/i', $error, $errorDst))
							{
								$errorLine = htmlspecialchars(substr($input_lines[$errorDst[1]], 0, $errorDst[2])).
									'<strong>'.htmlspecialchars($input_lines[$errorDst[1]][$errorDst[2]]).'</strong>'.
									htmlspecialchars(substr($input_lines[$errorDst[1]], $errorDst[2] + 1));
							}
							else $errorLine = '(Line not found)';
							
							$errors .= '<li>'.htmlspecialchars($error).
								'<br/><code>'.$errorLine.'</code></li>';
						}
					}
					
					if($errors == '<ul>')
						return true;
						
					$errors .= '</ul>'.cms_functions::string('libfunctions.tidy');
					
					return $errors;
				}
				else 
				{
					return true;
				}
			}
			else
			{
				// Thanks to htmlhelp.com - your validation service is great :)
				$validatorSocket = fsockopen('www.htmlhelp.com', '80', $errNo, $errStr, 30);
				if(!$validatorSocket)
				{
					trigger_error(cms_functions::string('libfunctions.socketfailed'), E_USER_WARNING);
					return false;
				}
			
				$validatorData = 'area='.urlencode($validator_input).'&input=yes&warnings=yes';
				fputs($validatorSocket, 'POST /cgi-bin/validate.cgi HTTP/1.0'."\r\n".
									'Host: www.htmlhelp.com'."\r\n".'Connection: close'."\r\n".
									'Content-length: '.strlen($validatorData)."\r\n\r\n".$validatorData);
			
				$validatorOutput = '';			
				while(!feof($validatorSocket))
					$validatorOutput .= fgets($validatorSocket);
				
				fclose($validatorSocket);
			
				// Get validation errors
				if(!preg_match('/<h2>Errors and Warnings<\/h2>\s+(<ul>.+?<\/ul>)/si', $validatorOutput, $strippedValidatorOutput))
					return true;
				else 
				{
					// Filter line numbers and links
					$strippedValidatorOutput[1] = preg_replace('/\s+<strong>\^<\/strong>(<\/pre>)/', '$1', $strippedValidatorOutput[1]);
					$strippedValidatorOutput[1] = preg_replace('/<a .*?>(.+?)<\/a>/', '$1', $strippedValidatorOutput[1]);
					$strippedValidatorOutput[1] = preg_replace('/(?<=<li>)(.+?)\s+(?=<pre>)/s', '<p>$1</p>', $strippedValidatorOutput[1]);
					$strippedValidatorOutput[1] = preg_replace('/<\/?pre>/i', '', $strippedValidatorOutput[1]);
					$strippedValidatorOutput[1] = preg_replace('/class=html/i', '', $strippedValidatorOutput[1]);
					
					// Return the error message
					return $strippedValidatorOutput[1].cms_functions::string('libfunctions.validator');
				}
			}
		}
		    
		// Redirect
		function redirect($args = array())
		{
			header('Location: '.str_replace('&amp;', '&', cms_functions::generate_url($args)));
		}
		
		function generate_url($args = array())
		{   	
			$url 	= 'index';
			$attach = '';
			foreach($args as $key => $value)
				if(CMS_USE_MOD_REWRITE && strpos($key, '-') === false && strpos($value, '-') === false && str_replace('+', ' ', urlencode($value)) == $value && str_replace('+', ' ', urlencode($key)) == $key)
					$url 	.= '-'.$key.'-'.$value;
				else 
					$attach .= urlencode($key).'='.urlencode($value).'&amp;';
					
			if(CMS_USE_MOD_REWRITE)
				$url .= '.html';
			else 
				$url .= '.php';
				
			if($attach != '')
				$url .= '?'.substr($attach, 0, -5);
				
			return $url;
		}
		
		// Return a parsed article
		function article_text($article)
		{    	
			$article_object = new cms_article($article);
			if(CMS_USE_WIKI_FORMATTING === true)
				$article_text	= cms_functions::parse_input(cms_functions::wiki_format($article_object->text));
			else 
				$article_text	= cms_functions::parse_input($article_object->text);
			return $article_text;
		}
		
		// Parse text templates before saving an article
		function parse_text_templates($input)
		{
			$replacements = array();
			
			// Username
			$replacements['/(^|\s|>)-- user(<|\s|$)/mi'] = '$1<address>-- <a href="'.
				cms_functions::generate_url(array('article' => 'author_'.$GLOBALS['cms_users']->currentUserNick)).'">'.
				$GLOBALS['cms_users']->currentUserNick.'</a> ('.date(cms_functions::string('generic.verylongdate')).')</address>$2';
				
			$input = preg_replace(array_keys($replacements), $replacements, $input);
			
			return $input;
		}
		
		// Wiki textformatting
		function wiki_format($text)
		{
			// Remove M$ Dos \r's before \n's
			// This is needed for compatibility with old versions of project_cms
			$text = "\n".str_replace("\r\n", "\n", $text);
			
			// NOWIKI tags
			$nowikiArrayID = uniqid(time());
			$GLOBALS['--nowiki'.$nowikiArrayID] = array();
			$text = preg_replace_callback('/<(nowiki|pre|code)[^>]*>(.+?)<\/\\1>/is', create_function('$arg',
				'$nowikiId = uniqid(time()); $GLOBALS["--nowiki'.$nowikiArrayID.'"]["--".$nowikiId."--"] = (strtolower($arg[1]) == "nowiki" ? $arg[2] : $arg[0]); return "--$nowikiId--";'), $text);
			
			// Paragraphs
			$text = '<p>'.str_replace("\n\n", "\n</p>\n\n<p>\n", $text).'</p>';
			
			// Tables
			$text = preg_replace_callback('/(^--+$\s?)(^.+$\s?)\\1((?:^\|.+$\s?)+)\\1/m', create_function('$arg',
				'
				$orginLength = strlen(trim($arg[1]));
				if(strlen(trim($arg[2])) != $orginLength) return $arg[0];
				$retVal = "</p>\n<table summary=\"\">\n".
				   "\t<tr>";
				foreach(explode("|", $arg[2]) as $heading)
					$retVal .= "<th>".trim($heading)."</th>";
				$retVal .= "</tr>\n";
				$arg[3] = explode("\n", $arg[3]);
				foreach($arg[3] as $row)
				{
					if(!$row) continue;				
					if(strlen(trim($row)) != $orginLength) return $arg[0];
					$retVal .= "\t<tr>";
					foreach(explode("|", substr($row, 1, -1)) as $col)
						$retVal .= "<td>".trim($col)."</td>";
					$retVal .= "</tr>\n";
				}
				
				$retVal .= "</table>\n<p>\n";
				$retVal = preg_replace(array("#<td>\s*</td>#", "#<th>\s*</th>#", "#<tr>\s*</tr>#"), "", $retVal);

				return $retVal;
				'), $text);
			
			// Headings
			$text = preg_replace('/^={4}(.+)={4}$/m', "</p>\n<h4>$1</h4>\n<p>", $text);
			$text = preg_replace('/^={3}(.+)={3}$/m', "</p>\n<h3>$1</h3>\n<p>", $text);
			$text = preg_replace('/^={2}(.+)={2}$/m', "</p>\n<h2>$1</h2>\n<p>", $text);
			
			// Lists
			$text = preg_replace('/^(\*+) (.+)$/me', 'str_repeat("<ul><li>", strlen(\'$1\')).\'$2\'.str_repeat("</li></ul>", strlen(\'$1\'))', $text);
			while(preg_match('/(?:<\/ul>\s<ul>|<\/li>\s<li><ul>)/', $text))
			{
				$text = preg_replace('/<\/ul>(\s)<ul>/', '$1', $text);
				$text = preg_replace('/<\/li>(\s)<li><ul>/', '$1<ul>', $text);
			}
			
			// Paragraphs before and after lists
			$text = preg_replace_callback('/<\/?ul>/i', create_function('$arg',
				'static $ulCtr; $isOpen = strlen($arg[0]) == 4; $ulCtr += $isOpen ? 1 : -1;
				 if($ulCtr == 1 && $isOpen) return "</p><ul>"; if($ulCtr == 0 &! $isOpen) return "</ul><p>"; return
				 $arg[0];'), $text);
				
			// Indented code
			$text = preg_replace('/(?:^ .+$\s?)+/m', "</p><pre>\n$0\n</pre><p>", $text);
			
			// Horizontal line
			$text = preg_replace('/^---+$/m', "\n</p><hr/><p>\n", $text);
			
			// Images
			$text = preg_replace('/\[\[image:(.+?)\|(.+?)\]\]/i', '<img src="$1" alt="$2"/>', $text);
			$text = preg_replace('/\[\[image:(.+?)\]\]/i', '<img src="$1" alt=""/>', $text);
			
			// URLs
			$text = preg_replace('/\[\[(.+?)\|(.+?)\]\]/', '<a href="$1">$2</a>', $text);
			$text = preg_replace('/\[\[(.+?)\]\]/', '<a href="$1">$1</a>', $text);
			$text = preg_replace('/http:\/\/\S+/', '<a href="$0">$0</a>', $text);
			$text = preg_replace('/(?<=\s|^)www\.\S+/', '<a href="http://$0">$0</a>', $text);
			
			// Bold & italic
			$text = preg_replace("/'''''(.+?)'''''/", '<strong><em>$1</em></strong>', $text);
			
			// Bold
			$text = preg_replace("/'''(.+?)'''/", '<strong>$1</strong>', $text);
			
			// Italics
			$text = preg_replace("/''(.+?)''/", '<em>$1</em>', $text);
			
			// Remove empty paragraphs
			$text = preg_replace('/<p>\s+<\/p>/', '', $text);
			
			// & characters without a meaning ... :D
			$text = preg_replace('/&(?!\S+;)/', '&amp;', $text);
			
			// Write NOWIKI tags back to the text
			$text = str_replace(array_keys($GLOBALS['--nowiki'.$nowikiArrayID]), $GLOBALS['--nowiki'.$nowikiArrayID], $text);
			unset($GLOBALS['--nowiki'.$nowikiArrayID]);
			
			return $text;
		}
	
		// Parsing of articles (e.g. highlighting)
		function parse_input($input)
		{			
			// Replace code and pre tags inside of code/pre tags
			// To do that, insert "cms_replaced_sub_" in front of those tags
			while(preg_match('/<(code|pre)([^>]*)>((?:.(?!<\/\\1>))+)<\\1([^>]*)>(.+)<\/\\1>((?:.(?!<\\1>))+)<\/\\1>/siU', $input))
			$input = preg_replace('/<(code|pre)([^>]*)>((?:.(?!<\/\\1>))+)<\\1([^>]*)>(.+)<\/\\1>((?:.(?!<\\1>))+)<\/\\1>/siU',
								  '<$1$2>$3<cms_replaced_sub_$1$4>$5<cms_replaced_sub_/$1>$6</$1>',
								  $input);
	
			// Highlight sourcecode inside of those tags
			$input = preg_replace_callback('/<code([^>]*)>\s*(.+)\s*<\/code>/isU', array('cms_functions', 'parse_input_highlight'), $input);
			$input = preg_replace_callback('/<pre([^>]*)>\s*(.+)\s*<\/pre>/isU', array('cms_functions', 'parse_input_highlight'), $input);
			
			// Deactivate script tags
			$input = preg_replace('/<(script.*)>/siSU', '&lt;$1&gt;', $input);
			
			// Replace templates
			$cms_templates_file = new cms_data_storage(CMS_DATA_PATH.'/templates');
			$cms_templates_data = &$cms_templates_file->data;
			foreach($cms_templates_data as $searchFor => $replaceBy)
				$input = preg_replace('/'.str_replace('/', '\\/', $searchFor).'/si', $replaceBy, $input);
			
			return $input;
		}
		
		// The highlighting
		// Uses the highlighting class from www.pberndt.com
		function parse_input_highlight($input)
		{
			static $codeParser;
			
			// Init the highlighting function
			if(!$codeParser)
			{
				require_once(CMS_INCLUDE_PATH.'/highlight_code.php');
				$codeParser = new highlighter();
				for($highlightID=3;$highlightID>0;$highlightID--)
					$codeParser->add_default_keywords($highlightID);
			}
			
			$codeParser->inPhpBB(false);
			
			// Rewrite subelements
			$highlighed_code = $codeParser->parse(trim(str_replace('<cms_replaced_sub_', '<', $input[2])));
			
			// Generate line numbers
			$lines = '';
				foreach(split("\n", $highlighed_code) as $line)
				$lines .= (str_pad(++$lineNumber, 3, '0', STR_PAD_LEFT).'<br/>');
			
			// Write attributes
			if(preg_match('/class=(\'|")/i', $input[1]))
				$input[1] = preg_replace('/class=(\'|")/i', '$0highlight ', $input[1]);
			else 
				$input[1] .= ' class="highlight"';
				
			// Return code depending on the HTML element
			if(strtolower(substr($input[0], 0, 5)) == '<code')	        		
				return	'<code'.$input[1].'>'.$highlighed_code.'</code>';
			else 
			{
				return 	'<div class="sourcecode"><div class="line_numbers">'.$lines.'</div>'.
						'<code'.$input[1].'>'.$highlighed_code.'</code></div>';
			}
		}
				
		// A better dir() function which returns an array
		function get_dir($dir, $pattern = false, $order = 'none')
		{
			$retVal = array();
			$dir_object = @dir($dir);
			if(!$dir_object) return $retVal;
			while(is_string($file = $dir_object->read()))
				if($file != '.' && $file != '..' && ($pattern === false || preg_match($pattern, $file, $ausgabe)))
					$retVal[$dir.'/'.$file] = isset($ausgabe[1]) ? $ausgabe[1] : $file;
					
			switch($order)
			{
				case 'none':
					$retVal = array_values($retVal);
					break;
					
				case 'alpha':
					sort($retVal, SORT_STRING);
					break;
				
				case 'nat':
					natcasesort($retVal);
					break;
					
				case 'time':	
					$fileList = array();
					foreach($retVal as $file => $data)
					{
						$ctr = 0;
						while(isset($fileList[filemtime($file)+(++$ctr)]));
						$fileList[filemtime($file)+$ctr] = $data;
					}
					krsort($fileList, SORT_NUMERIC);
					$retVal = array_values($fileList);
					break;
					
				default:
					trigger_error(sprintf(cms_functions::string('libfunctions.unknownorder'), $order), E_USER_WARNING);
			}
			
			return $retVal;
		}
		
		// Remove whitespace from a HTML file
		function html_strip_whitespace($text)
		{
			// Don't do that inside of code/pre tags
			$codetagArrayID = uniqid(time());
			$GLOBALS['--codetag'.$codetagArrayID] = array();
			$text = preg_replace_callback('/<(pre|code)[^>]*>(.+?)<\/\\1>/is', create_function('$arg',
				'$codetagId = uniqid(time()); $GLOBALS["--codetag'.$codetagArrayID.'"]["--".$codetagId."--"] = (strtolower($arg[1]) == "codetag" ? $arg[2] : $arg[0]); return "--$codetagId--";'), $text);
			
			// Remove whitespace
			$text = preg_replace('/(\s)\s*/', '$1', $text);
			
			// Rewrite code/pre tags into the code
			$text = str_replace(array_keys($GLOBALS['--codetag'.$codetagArrayID]), $GLOBALS['--codetag'.$codetagArrayID], $text);
			unset($GLOBALS['--codetag'.$codetagArrayID]);
			
			return $text;
		}
		
		// Check whether a file is called directly or if it's inside the CMS
		function inside_cms()
		{
			return true;
		}
		
		// Return a string in the template's language
		function string($name)
		{
			static $lang;
			if(empty($lang))
			{
				// I can't use "return" in the language file because of a bug
				// in PHP < 4.3
				require(CMS_TEMPLATE_PATH.'/language.php');
				$lang = $cms_language;
			}
			
			if(!isset($lang[$name]))
				trigger_error('Language string '.$name.' missing', E_USER_WARNING);
			
			return $lang[$name];
		}
		
		// Generate keywords for the current $pageContent
		// Usage:
		// <meta name="keywords" content="$code:{cms_functions::generate_keywords($pageContent)}" />
		function generate_keywords($text)
		{
			$keywords = array();
			
			// Count words in Hx Tags
			if(preg_match_all('/<h([1-5])>(.+?)<\/h\\1>/i', $text, $headers))
			{
				foreach($headers[2] as $key => $header)
					if($words = preg_split('/\s+/', preg_replace('/<[^>]+>/', '', $header)))
						foreach($words as $word)
							$keywords[$word] += 7 - $headers[1][$key];
			}
			
			// Strip tags
			$text = preg_replace('/<[^>]+>/', '', $text);
			
			// Count words
			if(preg_match_all('/\w+/i', $text, $words))
			{
				foreach($words[0] as $word)
					$keywords[strtolower($word)]++;
			}
			
			// Remove short keywords and keywords with less than 2 matches
			if($keywords)
			foreach(array_keys($keywords) as $keyword)
				if(strlen($keyword) < 4)
					unset($keywords[$keyword]);
			
			if($keywords)
			foreach($keywords as $keyword => $count)
				if($count  < 2)
					unset($keywords[$keyword]);
					
			if($keywords)
			arsort($keywords, SORT_ASC);
			
			return implode(', ', array_keys($keywords));
		}
		
		// Error handling
		function error_catch($errno, $errstr, $errfile, $errline)
		{
			if($errno != E_USER_ERROR && $errno != E_USER_WARNING && $errno != E_ERROR && $errno != E_WARNING)
				return true;
				
			// Print a error message
			if(is_object($GLOBALS['cms_cache']))
				$GLOBALS['cms_cache']->set_dont_cache_flag();
			while(ob_get_level())
				ob_end_clean();
				
			$pageTitle		= cms_functions::string('libfunctions.pagetitleerror');
			$pageContent 	= sprintf(cms_functions::string('libfunctions.pageerrortext'), htmlspecialchars($errstr));
			
			// Give detailled information to admins only
			if(is_object($GLOBALS['cms_users']) && $GLOBALS['cms_users']->checkPermission('admin') === true)
			{
				// Errortype
				$errConstantType = 'E_UNKNOWN';
				foreach(get_defined_constants() as $constant => $value)
					if($value == $errno &! strncmp($constant, 'E_', 2))
						$errConstantType = $constant; 
				$traceBack = '';
				
				// Backtrace
				foreach(array_reverse(debug_backtrace()) as $traceItem)
				{
					$traceBack .= $traceItem['class'].$traceItem['type'].$traceItem['function'].'(';
					if(is_array($traceItem['args']) &! empty($traceItem['args']))
					{
						foreach($traceItem['args'] as $value)
						{
							if(is_string($value)) $traceBack .= "'".$value."', ";
							elseif(is_object(($value))) $traceBack .= '$object, ';
							elseif(is_array(($value))) $traceBack .= '$array, ';
							else $traceBack .= $value.', ';
						}
						$traceBack = substr($traceBack, 0, -2);
					}
					$traceBack .= ");\n";
				}
				
				// Code
				$fromLine 	= max(0, $errline - 3);
				$errCode = '';
				foreach(array_slice(file($errfile), $fromLine, 6) as $errCodeLine)
					$errCode .= trim($errCodeLine)."\n";
				
				$pageContent	.= sprintf(cms_functions::string('libfunctions.errortext2'),
									htmlspecialchars($errfile.':'.$errline),
									htmlspecialchars($errno.' ('.$errConstantType.')'),
									cms_functions::parse_input('<code>'.$traceBack.'</code>'),
									cms_functions::parse_input('<code>'.$errCode.'</code>'));
			}
			
			if(!is_object($GLOBALS['cms_tpl']))
				die($pageContent);
			else
				die(eval($GLOBALS['cms_tpl']->tpl('index')));
		}
	}
	
	// Init errorhandling
	set_error_handler(array('cms_functions', 'error_catch'));
	
	
	
	// Various functions and definitions for old PHP versions
	if(!function_exists('array_walk_recursive'))
	{
		function array_walk_recursive(&$input, $function_name, $userdata = null)
		{
			foreach($input as $key => $value)
				if(is_array($value))
					array_walk_recursive(&$input[$key], $function_name, $userdata);
				else
					$function_name($input[$key], $key, $userdata);
		}
	}
	
	if(!function_exists('file_put_contents'))
	{
		define('FILE_APPEND', 8);
		function file_put_contents($file, $data, $flags = null, $context = null)
		{
			$file_context = fopen($file, ($flags == FILE_APPEND ? 'a' : 'w').'b');
			flock($file_context, LOCK_EX);
			fwrite($file_context, $data);
			flock($file_context, LOCK_UN);
			fclose($file_context);
		}
	}
	
	if(!function_exists('file_get_contents'))
	{
		function file_get_contents($file)
		{
			return implode('', file($file));
		}
	}
	
	if(!function_exists('stripos'))
	{
		function stripos($haystack, $needle, $offset = 0)
		{
			return strpos(strtolower($haystack), strtolower($needle), $offset);
		}
	}
	
	if(!defined('PHP_EOL'))
		define('PHP_EOL', "\n");
		
?>