<?php

	/*
		Project_CMS

		Library for templates
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	

	class cms_template_engine
	{
		var $templateCache;
		var $globalAccess;
		var $templatePath;
		var $templateData;
		
		// Constructor
		function cms_template_engine($templatePath, $globalReference)
		{
			$this->templatePath = $templatePath.(substr($templatePath, -1) == '/' ? '' : '/');
			$this->globalAccess = '$GLOBALS["'.$globalReference.'"]';
			$this->templateData = array();
		}

		
		// Load a template
		function loadTemplate($templateName)
		{
			$suffixes = array('.html', '.htm');
			$suffixID = 0;
			while(!file_exists($this->templatePath.$templateName.$suffixes[$suffixID]))
				if(++$suffixID == count($suffixes))		
					trigger_error(sprintf(cms_functions::string('libtemplates.notfound'), $templateName), E_USER_ERROR);
				
			$templateData = file_get_contents($this->templatePath.$templateName.$suffixes[$suffixID]);
			if(get_magic_quotes_runtime())
				$templateData = stripslashes($templateData);
			$templateData = addcslashes($templateData, '\\"');
			$this->templateCache[$templateName]['rawData']  = $templateData;
			$this->templateCache[$templateName]['path']		= $templateName;
			$this->templateCache[$templateName]['phpPath']	= '["'.$templateName.'"]';
			$this->loadTemplate_createPath($this->templateCache[$templateName]);
		}
		
		function loadTemplate_createPath(&$templateData)
		{
			// Load childtemplates
			preg_match_all('/<!-- #template (.+?) -->(.+?)\s*<!-- #end \\1 -->/si', $templateData['rawData'],
				$templateMatches, PREG_SET_ORDER);
				
			foreach($templateMatches as $templateMatch)
			{
				$childTemplate = &$templateData['childTemplates'][$templateMatch[1]];
				if($childTemplate)
					trigger_error(sprintf(cms_functions::string('libtemplate.parserror'), $templateData['path'],
						$templateMatch[1]), E_USER_ERROR);
					
				$childTemplate['path'] 		= $templateData['path'].'.'.$templateMatch[1];
				$childTemplate['phpPath']	= $templateData['phpPath'].'["sub"]["'.$templateMatch[1].'"]';
				$childTemplate['rawData'] 	= $templateMatch[2];
				
				$this->loadTemplate_createPath($childTemplate);
			}
			
			// Replace childtemplates by parsed data
			$templateData['data'] = preg_replace(
				'/\s*<!-- #template (.+?) -->.+?<!-- #end \\1 -->/si',
				'".'.$this->globalAccess.'->templateData'.$templateData['phpPath'].'["data"]["'.$templateData['path'].'.\\1"]."',
				$templateData['rawData']);
				
			// Indent variables in the output
			$templateData['data'] = preg_replace(
				'/^((?:(?!\n)\s)+)\$([A-Z0-9_:\[\]\->{}()\'\']+)(\s*)$/msi',
				// The str_replace is needed for compatibility with old versions of project_cms
				'$1".@preg_replace("/(?:\r?\n|\n?\r)/", "\n$1", "$$2")."$3',
				$templateData['data']);
				
			// Now for the other functions:
			// HTMLSpecialchars
			$templateData['data'] = preg_replace(
				'/\$html:([A-Z0-9_\[\]\->]+)/si',
				'".@htmlspecialchars($\\1)."',
				$templateData['data']);
			// Code
			$templateData['data'] = preg_replace(
				'/\$code:\{([^\}]+)\}/sie',
				'\'".(\'.@stripcslashes(\'\\1\').\')."\'',
				$templateData['data']);
			// Textarea
			$templateData['data'] = preg_replace(
				'/\$noindent:([A-Z0-9_\[\]\->]+)/si',
				// The str_replace is needed for compatibility with old versions of project_cms
				'".@str_replace(array("\r\n", "\n"), "&#013;", htmlspecialchars($\\1))."',
				$templateData['data']);
			
				
		}
		
		// Useraccess on a template
		function tpl($templateName)
		{
			$templatePath = explode('.', $templateName);
			
			if(!isset($this->templateCache[$templatePath[0]]))
				$this->loadTemplate($templatePath[0]);
				
			// In childtemplates, get the parente element
			if(count($templatePath) > 1)
			{
				$parentTemplate = array_slice($templatePath, 0, count($templatePath) - 1);
				$parentTemplateData = &$this->templateCache[array_shift($parentTemplate)];
				if($parentTemplate)
				foreach($parentTemplate as $parent)
					$parentTemplateData = &$parentTemplateData['childTemplates'][$parent];
			}
			
			// Follow the path of the templates
			$returnTemplate = &$this->templateCache[array_shift($templatePath)];
			if($templatePath)
			foreach($templatePath as $child)
				$returnTemplate = &$returnTemplate['childTemplates'][$child];
			
			if(!$returnTemplate)
				trigger_error(sprintf(cms_functions::string('libtemplates.notfound'), $templateName), E_USER_ERROR);
			
			// Now thing which apply on any template:
			$tmpRetVal = '$retVal'.uniqid(time());
			$returnData = $tmpRetVal.' = "'.$returnTemplate['data'].'";
							'.$this->globalAccess.'->templateData'.
							$returnTemplate['phpPath'].' = array(); ';
				
			// Do things based on the template level:
			if(!isset($parentTemplateData))
				// Level 0 -> Output
				$returnData .= 'return '.$tmpRetVal.';';
			else
				// Level > 0 -> Save 
				$returnData .= $this->globalAccess.'->templateData'.$parentTemplateData['phpPath'].
						'["data"]["'.$returnTemplate['path'].'"] .= '.$tmpRetVal.';';
			
			return $returnData;
		}
	}
	
	// Init class
	$cms_tpl = new cms_template_engine(CMS_TEMPLATE_PATH.'/', 'tpl');
?>