<?php

	/*
		Project_CMS

		Library for user management
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	

	class cms_users
	{
		var $userData_file;
		var $userData;
		var $currentUser;
		var $currentUserNick;
		
		function cms_users()
		{
			// Load userdata
			$this->userData_file 	= new cms_data_storage(CMS_USER_DATA_FILE);
			$this->userData 		= &$this->userData_file->data;
			
			// Login
			if(CMS_USE_HTTP_AUTHENTICATION === true)
			{
				// Use HTTP authentication
				settype($_SERVER['PHP_AUTH_USER'], 'string');
				$loginUser	= strtolower($_SERVER['PHP_AUTH_USER']);
				$loginPass	= sha1($_SERVER['PHP_AUTH_PW']);
			}
			else 
			{
				// Use sessions
				ini_set('arg_separator.output', '&amp;');				
				session_name('open_cms_id');
				
				// Let the session expire after 10 minutes
				session_cache_expire(10);
				
				// Start the session
				session_start();
				
				// When using sessions, check if the IP is valid
				// I know that this is nasty for AOL users, but others
				// will be pleased about that...
				if(isset($_SESSION['ip']))
					if($_SESSION['ip'] !== $_SERVER['REMOTE_ADDR'])
					{
						// For security reasons, close the session and destroy data
						session_destroy();
						unset($_SESSION);
					}
				else 
					$_SESSION['ip'] = $_SERVER['REMOTE_ADDR'];
					
				// Is the user logged in?
				if(isset($_SESSION['loginData']))
				{
					// Load data
					$loginUser = $_SESSION['loginData']['user'];
					$loginPass = $_SESSION['loginData']['pass'];
				}
				
				// Handle login formular
				if(isset($_REQUEST['submitlogin']))
				{
					// Load login
					$loginUser = $_POST['login'];
					$loginPass = sha1($_POST['password']);
					
					// Save login into session
					$_SESSION['loginData']['user'] = $loginUser;
					$_SESSION['loginData']['pass'] = $loginPass;
					
					// Save login for 3 months into a cookie
					setcookie('project_cms_login', $loginUser, time() + 24*3600*90);
					$_COOKIE['project_cms_login'] = $loginUser;
				}
			}
			
			// Check login
			if(isset($this->userData[$loginUser]))
			{
				if($this->userData[$loginUser]['pass'] == $loginPass)
				{
					$this->currentUserNick  = $loginUser;
					$this->currentUser 		= &$this->userData[$loginUser];
				}
				else 
				{
					$GLOBALS['cms_eventlog']->write_log(sprintf(cms_functions::string('libusers.loginfailed'), $loginUser), 'index.html');
					$this->currentUser = false;
					
					// If a session was started, destroy it as it's not needed
					if(session_id())
						session_destroy();
				}
			}
			else 
			{
				// If a session was started, destroy it as it's not needed
				if(session_id())
					session_destroy();
					
				$this->currentUser = false;
			}
		}
		
		function userList()
		{
			return $this->userData;
		}
		
		// Send an auth header
		function sendLoginRequired()
		{
			if($this->currentUser !== false) return false;
			
			header('HTTP/1.1 401 Unauthorized');
			header('WWW-Authenticate: Basic realm="Login im CMS"');
			
			return true;
		}
		
		/* -------------------------------------------------- */
		//
		// Permissions are saved as strings
		//
		function checkPermission($permissionName, $user = false)
		{
			if($user === false)
				$user = &$this->currentUser;
			else
				$user = &$this->userData[$user];
				
			// Check permission
			if($user === false) return false;
			if(isset($user['permissions']['admin'])) return true;
			return isset($user['permissions'][$permissionName]);
		}
		
		// Permission required
		function requirePermission($permissionName)
		{
			// Any page requiring login won't get cached
			$GLOBALS['cms_cache']->set_dont_cache_flag();
			
			if($this->checkPermission($permissionName))
				return true;
				
			if(CMS_USE_HTTP_AUTHENTICATION === true)
				$this->sendLoginRequired();
				
			$loginNick		= isset($_COOKIE['project_cms_login']) ? $_COOKIE['project_cms_login'] : '';
			$pageContent 	= eval($GLOBALS['cms_tpl']->tpl('error_403'));
			$pageTitle		= cms_functions::string('libusers.403title');
			die(eval($GLOBALS['cms_tpl']->tpl('index')));
		}
		
		function changePermission($permissionName, $value, $user = false)
		{
			if($user === false)
				$user = &$this->currentUser;
			else
				$user = &$this->userData[$user];
			
			if($value)
				$user['permissions'][$permissionName] = 1;
			else 
				if(isset($user['permissions'][$permissionName]))
				unset($user['permissions'][$permissionName]);
				
			$this->userData_file->save();
		}
		
		/* -------------------------------------------------- */
		
		function changePW($newPW, $user = false)
		{
			if($user === false)
				$user = &$this->currentUser;
			else
				$user = &$this->userData[$user];
				
			$user['pass'] = sha1($newPW);
			$this->userData_file->save();
		}
		
		function getInfo($infoName, $user = false)
		{
			if($user === false)
				$user = &$this->currentUser;
			else
				$user = &$this->userData[$user];
				
			return $user['info'][$infoName];
		}
		
		function changeInfo($infoName, $value, $user = false)
		{
			if($user === false)
				$user = &$this->currentUser;
			else
				$user = &$this->userData[$user];
				
			$user['info'][$infoName] = $value;
			$this->userData_file->save();
		}
	}
	
	// Init class
	$cms_users = new cms_users();
?>