<?php
	/*
		Project_CMS

		Mode: Edit a page
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	
	
	// Security check
	cms_functions::inside_cms();
	
	// Never cache this page (Not really needed here, but who knows...)
	$cms_cache->set_dont_cache_flag();
	
	if(!$_REQUEST['article'])
		$_REQUEST['article'] = 'index';
		
	$article = $_REQUEST['article'];
	
	// Permission for editing a page
	$cms_users->requirePermission('edit');
	if(!file_exists(CMS_DATA_PATH.'/article_'.urlencode($article)))
		$cms_users->requirePermission('new');
	
	if(!file_exists(CMS_DATA_PATH.'/article_'.urlencode($article)))
		trigger_error(cms_functions::string('generic.404'), E_USER_ERROR);
		
	$article_object	= new cms_article($article);
	
	// Permission for editing a locked page
	if($article_object->getLockedState())
		$cms_users->requirePermission('edit_locked');
		
	// Create an attachment
	if(isset($_REQUEST['upload']) && $_FILES['upload_file']['error'] == UPLOAD_ERR_OK)
	{
		$cms_users->requirePermission('edit_upload');
		$upload = &$_FILES['upload_file'];
		
		// If the attachment already existed, use the old filename
		$articleAttachments = $article_object->getAttachments();
		if(is_array($articleAttachments) && array_key_exists(basename($upload['name']), $articleAttachments))
		{
			$fileArray = $articleAttachments[basename($upload['name'])];
			
			list($dataFile, $dataLink) = array_values($fileArray);
			$fileName = CMS_PATH.'/'.$dataFile;
		}
		else 
		{
			// Filename
			$fileName = CMS_DATA_PATH.'/upload_'.urlencode($article).'_'.time().'_'.basename($upload['name']);
			
			$dataFile = substr($fileName, strlen(CMS_PATH) + 1);
			$dataLink = str_replace('%2F', '/', urlencode($dataFile));
		}
		
		// Deny uploading PHP files
		if(strtolower(substr($fileName, -4, 3)) == 'php' && strtolower(substr($fileName, -1)) != 's') $fileName .= 's';
		if(strtolower(substr($fileName, -3)) 	== 'php') $fileName .= 's';
		
		// Move upload
		if(!move_uploaded_file($upload['tmp_name'], $fileName))
			trigger_error(cms_functions::string('edit.uploadfailed'), E_USER_ERROR);		
		
		// Special files require special treatment
		if(stripos($upload['type'], 'image') === 0)
		{
			$imageInfo 			= getimagesize($fileName);
			$insertHyperlink 	= '<img src="'.$dataLink.'" width="'.$imageInfo[0].'" height="'.
									$imageInfo[1].'" alt="'.htmlspecialchars($upload['name']).'" />';
		}
		else 
		{
			$insertHyperlink	= '<a href="'.$dataLink.'" title="'.htmlspecialchars($upload['name']).' ('.
									round($upload['size'] / 1024, 2).' KB)">';
		}
		
		// Attach the upload to this article
		$article_object->newAttachment(basename($upload['name']), $dataFile, $insertHyperlink);
		
		// Write log
		$cms_eventlog->write_log(sprintf(cms_functions::string('edit.attachmentadded'), basename($upload['name']), $_REQUEST['article']),
			cms_functions::generate_url(array('article' => $_REQUEST['article'])));
	}
	if(isset($_REQUEST['deleteAttachment']))
	{
		$cms_users->requirePermission('edit_upload');
		$attachment = $article_object->getAttachments();
		$attachment = $attachment[$_REQUEST['deleteAttachment']];
		$article_object->deleteAttachment($_REQUEST['deleteAttachment']);
		unlink(CMS_PATH.'/'.$attachment['file']);
		cms_functions::redirect(array('article' => $_REQUEST['article'], 'mode' => 'edit'));
		
		// Write log
		$cms_eventlog->write_log(sprintf(cms_functions::string('edit.attachmentdeleted'), basename($upload['name']), $_REQUEST['article']),
			cms_functions::generate_url(array('article' => $_REQUEST['article'])));
			
		die();
	}
	
	// Change the article
	if(isset($_REQUEST['change']))
	{
		// Validate the input
		$validatorText = $_REQUEST['text'];
		$validatorText = cms_functions::parse_input($validatorText);
		if(CMS_USE_WIKI_FORMATTING === true)
			$validatorText = cms_functions::wiki_format($validatorText);
		
		// Validate
		$validation = cms_functions::validate_input($validatorText, CMS_CHARSET);
		
		if($validation === true)
		{				
			$articleText = cms_functions::parse_text_templates($_REQUEST['text']);
			// The str_replace is needed for compatibility with old versions of project_cms
			$articleText = str_replace(array("\r\n", "\r"), "\n", $articleText);
			$article_object->updateText($articleText, $cms_users->currentUserNick, $_REQUEST['comment']);
			cms_functions::redirect(array('article' => $_REQUEST['article']));
			
			// Wrtie log
			$cms_eventlog->write_log(sprintf(cms_functions::string('edit.changedversion'), $_REQUEST['article'], count($article_object->getChangeLog())),
				cms_functions::generate_url(array('article' => $_REQUEST['article'])));
		
			// Clear cached version of the page
			$cms_cache->clear_cache('article='.$_REQUEST['article']);
			if($_REQUEST['article'] == 'index')
				$cms_cache->clear_cache('index.php-.html');
			
			// Clear cache for RSS feet
			$cms_cache->clear_cache('mode=rss');
			
			die();
		}
		
		eval($cms_tpl->tpl('edit.error'));
	}
	else 
	{
		// Output editing page
		$comment		= '';
		$validation 	= '';
	}
	
	$comment 		= isset($_REQUEST['comment']) 	? $_REQUEST['comment'] 	: '';
	$article_text 	= isset($_REQUEST['text'])		? $_REQUEST['text'] 	: $article_object->text;

	// Change the rating status of the page
	if(isset($_REQUEST['votings']))
	{
		$article_object->setVotingsState($_REQUEST['votings']);
		
		// Write log
		$cms_eventlog->write_log(sprintf(cms_functions::string('edit.changedrating'), $_REQUEST['article'], ($_REQUEST['votings'] ? cms_functions::string('edit.on') : cms_functions::string('edit.off'))),
			cms_functions::generate_url(array('article' => $_REQUEST['article'], 'mode' => 'edit')));
		
		// Remove existing ratings
		if($_REQUEST['votings'] && file_exists(CMS_DATA_PATH.'/article_rating_'.urlencode($_REQUEST['article'])))
			unlink(CMS_DATA_PATH.'/article_rating_'.urlencode($_REQUEST['article']));
		
		// Delete cached versions of the current article
		$cms_cache->clear_cache('article='.$_REQUEST['article']);
		if($_REQUEST['article'] == 'index')
			$cms_cache->clear_cache('index.php-.html');
		
		// Clear cache for RSS feet
		$cms_cache->clear_cache('mode=rss');
			
		cms_functions::redirect(array('article' => $_REQUEST['article'], 'mode' => 'edit'));
		die();
	}
	
	// Change the locked-status of the page
	if(isset($_REQUEST['lock']))
	{
		// .. but only if permitted to do so ;)
		$cms_users->requirePermission('edit_locked');
		
		// Change modus
		$article_object->setLockedState($_REQUEST['lock']);
		
		// Write log
		$cms_eventlog->write_log(sprintf(cms_functions::string('edit.lockstate'), $_REQUEST['article'], $_REQUEST['lock'] ? cms_functions::string('edit.locked') : cms_functions::string('edit.unlocked')),
			cms_functions::generate_url(array('article' => $_REQUEST['article'], 'mode' => 'edit')));
		
		cms_functions::redirect(array('article' => $_REQUEST['article'], 'mode' => 'edit'));
		die();
	}
	
	// User ratings
	$votings = $article_object->getVotingsState() ? 0 : 1;
	$votings_text 	= $votings == 0 ? cms_functions::string('edit.permitted') : cms_functions::string('edit.prohibited');
	
	// Lock status
	$locked			= $article_object->getLockedState() ? 0 : 1;
	$locked_text	= $locked == 0 ? cms_functions::string('edit.locked') : cms_functions::string('edit.unlocked');

	$article_author	= &$article_object->author;
	$article_date	= date(cms_functions::string('generic.date'), $article_object->changeLog[count($article_object->changeLog)-1]['time']);
	if($changelog = $article_object->getChangeLog())
	foreach($changelog as $changeID => $change)
	{
		$changeID++;
		$change_user	= $change['user'];
		$change_comment	= $change['comment'];
		$change_time	= date(cms_functions::string('generic.longdate'), $change['time']);
		
		eval($cms_tpl->tpl('edit.change'));
	}
	if($attachments = $article_object->getAttachments())
	foreach($attachments as $attachmentFile => $attachmentData)
	{
		$attachmentHyperlink 	= $attachmentData['link'];
		$attachmentLink 		= str_replace('%2F', '/', urlencode($attachmentData['file']));
		
		eval($cms_tpl->tpl('edit.attachment'));
	}
	
	$pageContent = eval($cms_tpl->tpl('edit'));
	$pageTitle	 = sprintf(cms_functions::string('edit.pagetitle'), $article);
?>