<?php
	/*
		Project_CMS

		Mode: User management
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	
	
	// Security check
	cms_functions::inside_cms();
	
	// Never cache this page (Not really needed here, but who knows...)
	$cms_cache->set_dont_cache_flag();
	
	$cms_users->requirePermission('admin');

	
	$permissions = array('admin'			=> cms_functions::string('usermanagement.admin'),
						 'delete'			=> cms_functions::string('usermanagement.delete'),
						 'edit'				=> cms_functions::string('usermanagement.edit'),
						 'new'				=> cms_functions::string('usermanagement.new'),
						 'edit_locked'		=> cms_functions::string('usermanagement.edit_locked'),
						 'edit_upload'		=> cms_functions::string('usermanagement.edit_upload'), 
						 'view_eventlog'	=> cms_functions::string('usermanagement.view_eventlog'));

	// Accept changes
	// Here - and only here - the property gets accessed directly
	// This is _much_ more easy, but ugly as well ;)
	if(isset($_REQUEST['change']))
	{
		// Create a new user database
		foreach($_REQUEST['user'] as $userData)
		{
			$old_userID = $userData['orginal_nick'];
			$userID		= strtolower($userData['nick']);
			$perm		= $userData['permissions'];
			$delete		= $userData['delete'];
			$resetPW	= $userData['resetPW'];
			
			// Protect superadmin 
			if(CMS_ADMIN_USER === $old_userID || CMS_ADMIN_USER === $userID)
					continue;
		
			// Delete a user
			if($delete && isset($cms_users->userData[$old_userID]))
			{				
				$cms_eventlog->write_log(sprintf(cms_functions::string('usermanagement.deleted'), $old_userID), cms_functions::generate_url(array('mode' => 'usermanagement')));
				if(file_exists(CMS_DATA_PATH.'/article_author_'.$old_userID))
				{
					$article_object	= new cms_article('article_author_'.$old_userID);
	
					if($attachments = $article_object->getAttachments())
					foreach($attachments as $attachmentFile => $attachmentData)
						unlink(CMS_PATH.'/'.$attachmentData['file']);
	
					$article_object = null;
					unlink(CMS_DATA_PATH.'/article_'.urlencode('article_author_'.$old_userID));
				}
				unset($cms_users->userData[$old_userID]);
				continue;
			}

			if($old_userID != $userID && $old_userID != '')
			{
				$cms_eventlog->write_log(sprintf(cms_functions::string('usermanagement.renamed'), $old_userID, $userID), cms_functions::generate_url(array('mode' => 'usermanagement')));
				// Rename info files
				if(file_exists(CMS_DATA_PATH.'/article_author_'.$old_userID))
					rename(CMS_DATA_PATH.'/article_author_'.$old_userID, CMS_DATA_PATH.'/article_author_'.$userID);
				
				$cms_users->userData[$userID]['pass'] = $cms_users->userData[$old_userID]['pass'];
				unset($cms_users->userData[$old_userID]);
			}
			
			if($resetPW != '')
			{
				$cms_eventlog->write_log(sprintf(cms_functions::string('usermanagement.pwchanged'), $userID), cms_functions::generate_url(array('mode' => 'usermanagement')));
				$cms_users->changePW($resetPW, $userID);    			
			}
				
			unset($cms_users->userData[$userID]['permissions']);
			if($permissions);
			foreach($permissions as $permission => $desc)
				if(isset($perm[$permission]))
					$cms_users->userData[$userID]['permissions'][$permission] = 1;
		}
		
		// For security reasons, delete "0-users"
		unset($cms_users->userData['']);
		
		// Save
		$cms_users->userData_file->save();
		
		// And return
		cms_functions::redirect(array('mode' => 'usermanagement'));
		die();
	}
	
	// Columns
	foreach($permissions as $permissionName => $permissionDesc)
		eval($cms_tpl->tpl('edit_users.permissionname'));
		
	// Users
	$uId = 0;
	foreach($cms_users->userList() as $userName => $userData)
	{
		$isSuperAdmin = CMS_ADMIN_USER === $userName;
		foreach($permissions as $permissionName => $permissionDesc)
		{
			$permissionIsOn = isset($userData['permissions'][$permissionName]) ? 'checked="checked"' : '';
			eval($cms_tpl->tpl('edit_users.user.userpermission'));
		}
		eval($cms_tpl->tpl('edit_users.user'));
		$uId++;
	}
	$isSuperAdmin = false;
	
	// Mask: New user
	$userName = '';
	foreach($permissions as $permissionName => $permissionDesc)
	{
		$permissionIsOn = $permissionName == 'edit' ? 'checked="checked"' : '';
		eval($cms_tpl->tpl('edit_users.user.userpermission'));
	}
	eval($cms_tpl->tpl('edit_users.user'));
		
	$pageContent = eval($cms_tpl->tpl('edit_users'));
	$pageTitle	 = cms_functions::string('usermanagement.title');	
?>