<?php
	/*
		Project_CMS

		Mode: View ratings for an article
		
		Copyright (c) 2005 by Phillip Berndt
		
		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.
	
		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
	
		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	*/	
	
	// Security check
	cms_functions::inside_cms();
	if(!$article_object) die();
	
	// Load ratings for the current page
	$voting_file	= CMS_DATA_PATH.'/article_rating_'.urlencode($article);
	$voting_object 	= new cms_data_storage($voting_file);
	$voting_data	= &$voting_object->data;
	if(!isset($voting_data['percent']))
	{
		$voting_data['percent']  = 20;
		$voting_data['votes']	 = 1;
		$voting_data['iplock']	 = array();
		$voting_data['comments'] = array();
	}
		
	// Comment
	if(isset($_REQUEST['comment']))
	{
		// Check comment text
		// Replace whitespace - so we save space
		$commentText = $_REQUEST['text'];
		$commentText = preg_replace('/(\s)\s+/', '$1', $commentText);

		$commentNick = $_POST['nick'];
		
		// Check if the user may comment the article
		$may_comment = true;
		foreach($voting_data['comments'] as $comment)
			if($comment['ip'] === $_SERVER['REMOTE_ADDR'] && $comment['time'] + 500 >= time())
				$may_comment = false;
		$may_comment |= $cms_users->checkPermission('edit');
		
		// Save the comment
		if(trim($commentText) != '' && $may_comment)
		{
			// Create a new comment
			$newComment = &$voting_data['comments'][];
			
			// Insert
			$newComment['text'] = $commentText;
			$newComment['nick'] = $commentNick;
			$newComment['time'] = time();
			$newComment['ip']	= $_SERVER['REMOTE_ADDR'];
			
			// Delete any cached versions of the current page
			$cms_cache->clear_cache('article='.$_REQUEST['article']);
			if($_REQUEST['article'] == 'index' || $_REQUEST['article'] == '')
				$cms_cache->clear_cache('index.php-.html');
			
			// Write log
			$cms_eventlog->write_log(sprintf(cms_functions::string('rate.comment'), $_SERVER['REMOTE_ADDR']),
				cms_functions::generate_url(array('article' => $_REQUEST['article'])));
			
			// Save & return
			$voting_object->save();
			cms_functions::redirect(array('article' => $article));
			die();
		}
	}
	
	// Delete comments
	if(isset($_REQUEST['deleteComment']))
	{
		// This is allowed for users with "edit" rights - they area allowed to 
		// remove ratings anyway :)
		$cms_users->requirePermission('edit');
		
		// Delete
		if(isset($voting_data['comments'][$_REQUEST['deleteComment']]))
			unset($voting_data['comments'][$_REQUEST['deleteComment']]);
		
		// Clear cache for the current page
		$cms_cache->clear_cache('article='.$_REQUEST['article']);
		if($_REQUEST['article'] == 'index' || $_REQUEST['article'] == '')
			$cms_cache->clear_cache('index.php-.html');
		
		// Write log
		$cms_eventlog->write_log(sprintf(cms_functions::string('rate.commentDeleted')),
			cms_functions::generate_url(array('article' => $_REQUEST['article'])));
			
		// Save & return
		$voting_object->save();
		cms_functions::redirect(array('article' => $article));
		die();
	}
	
	// Rate
	if(isset($_REQUEST['percent']))
	{
		array_walk_recursive($voting_data['iplock'], create_function('$value, $key, &$orig', 'if($value + 500 < time()) unset($orig[$key]);'), $voting_data['iplock']);
		$may_vote		= !isset($voting_data['iplock'][$_SERVER['REMOTE_ADDR']]);
	}
	if(isset($_REQUEST['percent']) && $may_vote)
	{
		// Calculate new percentage value
		$voting_data['percent'] = ($voting_data['votes'] * $voting_data['percent']
									+ $_REQUEST['percent']) / ++$voting_data['votes'];
		
		// Set IP lock
		$voting_data['iplock'][$_SERVER['REMOTE_ADDR']] = time();
		
		// Delete page's cache
		$cms_cache->clear_cache('article='.$article);
		if($article == 'index' || $article == '')
			$cms_cache->clear_cache('index.php-.html');
			
		// Save & return
		$voting_object->save();
		cms_functions::redirect(array('article' => $article));
		die();
	}
	
	// View comments
	if(!empty($voting_data['comments']))
		foreach($voting_data['comments'] as $id => $comment)
		{
			$commentNick = $comment['nick'];
			$commentTime = date(cms_functions::string('generic.date'), $comment['time']);
			
			// Replace HTML special chars in $commentText
			$commentText = htmlspecialchars($comment['text']);
			$commentText = preg_replace("/^&gt;.+?(?=\r\n|\n\r|\n|\r|$)/m", '<cite>$0</cite>', $commentText);
			$commentText = preg_replace('/([a-z]+:\/\/)([a-z0-9-_\.]+)(\/[a-z0-9-_?&;#\.=+%\/]*)?/i', '<a href="$1$2$3">$2</a>', $commentText);
			$commentText = preg_replace("/(\r\n|\n\r|\n|\r)/", "<br/>\n", $commentText);
			$commentText = preg_replace_callback('/(?<=\s|^)[^<>&\s]{30,}(?=\s|$)/', create_function('$arg', 'return chunk_split($arg[0], 30, " ");'), $commentText);
			eval($cms_tpl->tpl('article.rating.comment'));
		}
	$userNick = isset($_COOKIE['nick']) ? $_COOKIE['nick'] : '';
			
	eval($cms_tpl->tpl('article.rating'));
?>